//   RW Sumomo
// ==============
// Demonstration of a Chibitsu-grade robowaifu
// -Part of the RW Foundations suite of libraries

// Filename: ./include/chibitsu/sumomo.hpp
// -Declarations of the baseline RW Sumomo project subsystem

#pragma once

#include "incls_all.hpp"

namespace rw::chibitsu {

//------------------------------------------------------------------------------
class Sumomo_b : public Chibitsu {
 public:
  Sumomo_b();                                    // default ctor
  explicit Sumomo_b(std::string const& handle);  // tag-parm'd ctor
};

//------------------------------------------------------------------------------
class Sumomo : public Sumomo_b {
 public:
  Sumomo();                                    // default ctor
  explicit Sumomo(std::string const& handle);  // tag-parm'd ctor

  auto preflight() -> bool override;

  inline auto flags() const -> Flags& { return flags_; }
  inline auto pompoms() const -> Pompoms& { return pompoms_; }
  inline auto tambourine() const -> Tambourine& { return tambourine_; }
  inline auto whistle() const -> Whistle& { return whistle_; }

 private:
  unique_ptr<Flags_b>      _flags_{make_unique<Flags_b>("sumomo_flags")};
  unique_ptr<Pompoms_b>    _pompoms_{make_unique<Pompoms_b>("sumomo_pompoms")};
  unique_ptr<Tambourine_b> _tambourine_{
      make_unique<Tambourine_b>("sumomo_tambourine")};
  unique_ptr<Whistle_b> _whistle_{make_unique<Whistle_b>("sumomo_whistle")};

  Flags&      flags_{reference_to<Flags>(_flags_.get())};
  Pompoms&    pompoms_{reference_to<Pompoms>(_pompoms_.get())};
  Tambourine& tambourine_{reference_to<Tambourine>(_tambourine_.get())};
  Whistle&    whistle_{reference_to<Whistle>(_whistle_.get())};
};

}  // namespace rw::chibitsu

//    --=^=--

// "EMERGENCY, EMERGENCY! THIS IS A CRISIS SITUATION!!"

// rw_chibitsu_lib.v211127
// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
