//  RW Foundations
// ================
// The RW Foundational Libraries

// Filename: ./include/incls_all.hpp
// -The following lines pull in the project's real *.hpp files.
// -note: this file also acts as a location for a few utility features

#pragma once

#include <cstdio>
#include <iostream>
#include <memory>
#include <sstream>
#include <stdexcept>
#include <string>

using std::cerr;
using std::cout;
using std::exception;
using std::make_unique;
using std::ostringstream;
using std::printf;
using std::puts;
using std::runtime_error;
using std::string;
using std::unique_ptr;

//
// NOTE: It is probably a good idea to be cautious about moving things around
// hereafter within this file; some dependencies here are sensitive to ordering.
//

//------------------------------------------------------------------------------
/** Our alias for a @c void*
 */
using Address = void*;

//------------------------------------------------------------------------------
/** Treat an @c Address as a reference to a general type @a T
 * @param[in] pt the type's @c Address to take the reference to
 * @return @a pt 's reference
 */
template <class T>
T& reference_to(Address pt) {
  return *static_cast<T*>(pt);
}

//------------------------------------------------------------------------------
#include "rw/rw_sb.hpp"  // RW class' fwd decl/alias

//------------------------------------------------------------------------------
/** Provides an identification structure for instantiated RW objects
 * @note: @b T ag, @b I d, @b P ointer
 *
 * This is simply a tagging-mechanism intended to be suitable for understanding
 * the potentially highly-complex callgraphs creatable inside deep RW (and
 * other) systems. IE, @a 'who-called-whom-and-who-answered-the-call?'.
 */
struct Tip {
  std::string   hdl{"nobody"};  // sentinel for an unassigned
  unsigned long id{9001'42};    //  "
  RW*           ptr{nullptr};   //  "

  friend bool operator==(Tip const& tip_a, Tip const& tip_b);
};

//------------------------------------------------------------------------------
/** Prints a Tip item's elements out to the console, with an optional message
 * @param[in] tip the Tip data struct to print
 * @param[in] msg optional description (&tc) string
 */
void prt_tag(Tip const& tip, std::string const& msg = "");

//------------------------------------------------------------------------------
inline void error(std::string const& s) {
  ostringstream ss{};
  ss << s << '\n';
  throw runtime_error(ss.str());
}

//------------------------------------------------------------------------------
inline void error(std::string const& err_msg,
                  Tip const&         t,
                  std::string const& func_nm = "") {
  ostringstream ss{};
  ss << err_msg << " ( from " << t.hdl << '.' << t.id << ", via " << func_nm
     << " )\n";
  throw runtime_error(ss.str());
}

//------------------------------------------------------------------------------
// THE RW 'SWITCHBOARD:A' - library class' fwd decls/aliases
//
#include "action/action_sb.hpp"
#include "animeyes/animeyes_sb.hpp"
#include "arigato/arigato_sb.hpp"
#include "atashi/atashi_sb.hpp"
#include "bones/bones_sb.hpp"
#include "chibitsu/chibitsu_sb.hpp"
#include "curator/curator_sb.hpp"
#include "dollhouse/dollhouse_sb.hpp"
#include "electro/electro_sb.hpp"
#include "gears/gears_sb.hpp"
#include "graphicom/graphicom_sb.hpp"
#include "hearsay/hearsay_sb.hpp"
#include "meido/meido_sb.hpp"
#include "shell/shell_sb.hpp"
#include "wired/wired_sb.hpp"

//------------------------------------------------------------------------------
// THE RW 'SWITCHBOARD:B' - library header includes
//
#include "rw/rw_incls_all.hpp"  // spcfcly. #include'd before other rw classes
//
#include "action/action_incls_all.hpp"
#include "animeyes/animeyes_incls_all.hpp"
#include "arigato/arigato_incls_all.hpp"
#include "atashi/atashi_incls_all.hpp"
#include "bones/bones_incls_all.hpp"
#include "chibitsu/chibitsu_incls_all.hpp"
#include "curator/curator_incls_all.hpp"
#include "dollhouse/dollhouse_incls_all.hpp"
#include "electro/electro_incls_all.hpp"
#include "gears/gears_incls_all.hpp"
#include "graphicom/graphicom_incls_all.hpp"
#include "hearsay/hearsay_incls_all.hpp"
#include "meido/meido_incls_all.hpp"
#include "shell/shell_incls_all.hpp"
#include "wired/wired_incls_all.hpp"

//    --=^=--

// "So is it true Sempai? Are robowaifus really better than any human woman?"

// rw_foundations_lib.v211127
// Copyright (2021)
// License MIT(Expat) https://opensource.org/licenses/MIT
